function [path2,descriptif,alpha,alpha_post,Q]=transcription(C,mode,K,harmoniques,distance,L,lissage)

% % % Cette fonction calcule la transcription en accords  partir du chromagramme
% % % Input :     
% % %     * C : chromagramme
% % %     * mode : mthode de dcodage 
% % %                 - 0 : par dfaut : majeur et mineur
% % %                 - 1 : on garde tous les types d'accords calculs en entre
% % %     * K : types d'accords pris en compte
% % %     * harmoniques : nombres d'harmoniques pris en compte
% % %     * distance : distances prises en compte
% % %     * L : tailles des voisinages pris en compte
% % %     * lissage : mthodes de lissage
% % %                 - 1 : filtre passe bas
% % %                 - 2 : filtre median
% % % Output :
% % %     * path : matrice contenant les diffrentes transcriptions
% % %     * descriptif : rappel des paramtres utiliss pour chaque transcription

                
%% Paramtres par dfaut

if nargin<7
    lissage=1;
end

if nargin<6
    L=1;
end

if nargin<5
    distance{1}='eu';
end

if nargin<4
    harmoniques=1;
end

if nargin<3
    K={'maj','min'};
end

if nargin<2
    mode=0;
end



niter=50;
display=0;
C=max(C,eps);


L=(L-1)/2;
[F,N]=size(C);

%% Calcul du nombre de simulations

N_harmoniques=length(harmoniques);
N_distance=length(distance);
N_L=length(L);
N_lissage=length(lissage);

N_simu=N_harmoniques*N_distance*N_L*N_lissage;

path=zeros(N,N_simu); % matrice contenant les rsultats de toutes les simulations
path2=zeros(N,N_simu);


silences=find(sum(C)<10^-4);


for i_harmoniques=1:N_harmoniques

%% Etape n1 : Gnration du dictionnaire d'accords

    N_K=length(K);
    W=zeros(12,12*N_K);

    for i_K=1:N_K
        pattern=find_pattern(K{i_K});
        pattern=create_harmonique(pattern,harmoniques(i_harmoniques));
        W(:,(i_K-1)*12+1:i_K*12)=create_pattern(pattern);
    end
    
W=max(W,10^-5);
W=norm1c(W);
    
   
    for i_distance=1:N_distance
        
%% Etape n2 : Calcul de la matrice de distance

       
        [critere,alpha,Q]=algo_EM_general(C,W,niter,display,distance{i_distance});

        
        critere(find(isnan(critere)))=0;

        
        for i_lissage=1:N_lissage
 
             for i_L=1:N_L

%% Etape n3 : Lissage (taille fixe)



[alpha_post]=lissage_L(critere,lissage(i_lissage),L(i_L)); 
                [garb,idx2]=max(alpha_post);
               
                path2=idx2(:);
                path2(silences)=24;
                
           
            end
        end
    end
end



switch mode
    
    case 0

    % Dans le cas mode=0 on ramne tous les accords dtects aux accords
    % majeur et mineur
    for i_K=1:N_K
    [pattern,quality]=find_pattern(K{i_K});
    path((path>=12*(i_K-1)+1)&(path<=i_K*12))=mod(path((path>=12*(i_K-1)+1)&(path<=i_K*12))-1,12)+quality*12;
    path2((path2>=12*(i_K-1)+1)&(path2<=i_K*12))=mod(path2((path2>=12*(i_K-1)+1)&(path2<=i_K*12))-1,12)+quality*12;
    end

    otherwise
        
        path=path-1;
        path2=path2-1;
end
        
 path(silences,:)=24;
 path2(silences,:)=24;






function [pattern,quality]=find_pattern(chord)

%%% Cette fonction renvoie la structure binaire associe  un type d'accord
 
    
pattern=zeros(12,1);

mode_vect=[0 1 0 0 0];

switch chord

    case 'maj'
        pattern([1 5 8])=1; 
        quality = 0;
    case 'min'
        pattern([1 4 8])=1;
        quality = 1;        
    case 'dim' 
        pattern([1 4 7])=1;
        quality = 2;    
    case 'aug' 
        pattern([1 5 9])=1;
        quality = 3;
    case 'maj7' 
        pattern([1 5 8 12])=1;
        quality = 0;
    case 'min7' 
        pattern([1 4 8 11])=1;
        quality = 1;
    case '7' 
        pattern([1 5 8 11])=1;
        quality = 0;        
    case 'dim7' 
        pattern([1 4 7 10])=1;
        quality = 2;
    case 'hdim7'
        pattern([1 4 7 11])=1;
        quality = 2;
    case 'minmaj7'
        pattern([1 4 8 12])=1;
        quality = 1;     
    case 'maj6'
        pattern([1 5 8 10])=1;
        quality = 0;        
    case 'min6' 
        pattern([1 4 8 10])=1;
        quality = 1;
    case '9'
        pattern([1 5 8 11 3])=1;
        quality = 0;          
    case 'maj9'
        pattern([1 5 8 12 3])=1;
        quality = 0;    
    case 'min9'
        pattern([1 4 8 11 2])=1;
        quality = 1;    
    case 'sus4'
        pattern([1 6 8])=1;
        quality = 4;
    case 'sus2'
        pattern([1 3 8])=1;
        quality = 4;

    otherwise
        error('Pas bon type d''accord')
end

quality=mode_vect(quality+1);











function [alpha_post,alpha,Q]=algo_EM_general(C,W,niter,display,method)

switch lower(method)
    case 'gau'
        sigma2=0.02;
        [alpha_post,alpha,Q]=algo_EM_gaussien(C,W,niter,display,sigma2);
    case 'poi'
        [alpha_post,alpha,Q]=algo_EM_poisson(C,W,niter,display);
    case 'gam'
        beta=3;
        [alpha_post,alpha,Q]=algo_EM_gamma(C,W,niter,display,beta);
            
    otherwise
        error('Mthode non reconnue');
end




function B=create_pattern(pattern)
%%% Cette fonction duplique et shifte le pattern d'accord pour chaque
%%% demi-ton
    
    B=zeros(12, length(pattern));
    B(:,1)=pattern;
    for i=2:12
        b=[B(end,i-1) ; B(:,i-1)];
        B(:,i)=b(1:12);
    end
    

function pattern_out=create_harmonique(pattern,nbharmo)
%%% Cette fonction calcule  partir d'un pattern de base, un pattern
%%% d'accord prenant en compte 1 ou plus harmoniques
h=0.6;
harmonique = [0 0 7 0 4 7];
poids=h.^(0:5);    

harmonique=harmonique(1:nbharmo);
poids=poids(1:nbharmo);

vector=find(pattern)-1;
N=length(vector);

pattern_out=zeros(12,1);

for k=1:N
    for i=1:12
        pattern_out(i)=pattern_out(i)+sum(poids(find(mod(harmonique+vector(k),12)+1==i)));
    end
end




function [critere]=lissage_L(D,lissage,L)
%%% Cette fonction calcule une version filtre d'une matrice de distances
%%% selon une mthode de lissage et une taille de voisinage
    
[N1,N2]=size(D);
critere=zeros(N1,N2);

switch lissage
    case 1
        %% Mthode de moyenne
        if L>0

            D2=[zeros(N1,L) D zeros(N1,L)];
            for n=1:N2
                   critere(:,n)=mean(D2(:,n:n+2*L),2);
            end

        else
            critere=D;
        end

    case 2
        %% Mthode de mediane
        if L>0
            critere=medfilt1(D,2*L+1,N2,2);
        else
            critere=D;
        end

    case 3
         %% Mthode filtre d'ordre
        if L>0

            D2=[zeros(N1,L) D zeros(N1,L)];
            for n=1:N2
                    d1=mean(D2(:,n:n+L-1),2);
                    d2=D2(:,n+L);
                    d3=mean(D2(:,n+L+1:n+2*L),2);
                    critere(:,n)=median([d1 d2 d3],2);
            end
        else
            critere=D;
        end

end




function n = norm1c(m)
%%% Cette fonction normalise une matrice selon les colonnes

if nargin < 1,error('Not enough input arguments.'); end

[mr,mc] = size(m);
if (mr == 1)
  n = ones(1,mc);
else
  n =ones(mr,1)*(ones./sum(m)).*m;
end



function [alpha_post,alpha,Q]=algo_EM_gamma(C,W,niter,display,beta)

if nargin<5
    beta=3;
end
if nargin<4
    display=0;
end
if nargin<3
    niter=200;
end

[M,N]=size(C);
[M,K]=size(W);

alpha=rand(K,1);
alpha=alpha/sum(alpha);

D=distance_itakura(C,W);

if display
Q=zeros(1,niter);
else
    Q=0;
end


P=calcule_gamma(D,C,beta);



for i=1:niter
    intermediaire=repmat(alpha,1,N).*P;
    alpha_post=intermediaire./(repmat(sum(intermediaire),K,1));
    
    if display
    Q(i)=-sum(sum(alpha_post.*(log(P+eps)+repmat(log(alpha+eps),1,N))));
    end
    alpha=sum(alpha_post,2)/sum(sum(alpha_post));
end

alpha=alpha';








function P=calcule_gamma(D,C,beta)
[M,N]=size(C);
[K,N]=size(D);
P=exp(-M*(log(gamma(beta))-beta*log(beta)+beta)-repmat(sum(log(C)),K,1)-beta*D);



function D=distance_itakura(C,W)

[M,N]=size(C);
[M,K]=size(W);
D=zeros(K,N);

for n=1:N
    C_n=repmat(C(:,n),1,K);
    D(:,n)=M*log(mean(C_n./W))-sum(log(C_n./W));
end







